@@ -19,6 +19,7 @@
 #include "../config.h"
 #endif
 
+#include <unistd.h>
 #include "ltris.h"
 #include "event.h"
 #include "chart.h"
@@ -32,10 +33,14 @@ extern Config config;
 extern char gametype_names[8][64];
 extern char gametype_ids[8][64];
 
+extern int display_w, display_h, video_forced_w, video_forced_h;
+
 int main(int argc, char *argv[])
 {
+	int c;
     int result = ACTION_NONE;
     int leave = 0;
+    const SDL_VideoInfo* info;
 
     /* i18n */
 #ifdef ENABLE_NLS
@@ -45,7 +50,7 @@ int main(int argc, char *argv[])
 #endif
     
     /* ltris info */
-    printf( "LTris %s\nCopyright 2002-2011 Michael Speck\nPublished under GNU GPL\n---\n", VERSION );
+    printf( "LTris %s\nCopyright 2002-2020 Michael Speck\nPublished under GNU GPL\n---\n", VERSION );
     printf( "Looking up data in: %s\n", SRC_DIR );
 #ifndef SOUND
     printf( "Compiled without sound and music\n" );
@@ -73,8 +78,38 @@ int main(int argc, char *argv[])
     strcpy(gametype_names[7],_("Vs CPU&CPU"));
     config_load();
 
+    while ( ( c = getopt( argc, argv, "wr:" ) ) != -1 )
+      {
+        switch (c)
+  	{
+  	case 'w': config.fullscreen=0; break;
+  	case 'r':
+  		sscanf(optarg, "%dx%d", &video_forced_w, &video_forced_h);
+  		printf("Trying to force resolution %dx%d\n",
+  				video_forced_w, video_forced_h);
+  		break;
+  	}
+      }
+
     init_sdl( SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_TIMER );
-    set_video_mode( std_video_mode( config.fullscreen ) );
+
+    info = SDL_GetVideoInfo();
+    printf("Display resolution: %d x %d\n", info->current_w, info->current_h);
+    display_w = info->current_w;
+    display_h = info->current_h;
+    if (video_forced_h > 0) {
+  	  if (video_forced_w > display_w || video_forced_h > display_h) {
+  		  printf("Forced resolution out of bounds, ignoring it\n");
+  		  video_forced_h = video_forced_w = 0;
+  	  }
+  	  if (video_forced_w < 640 || video_forced_h < 480) {
+  		  video_forced_w = 0;
+  		  video_forced_h = 0;
+  	  }
+    }
+
+    set_video_mode( config.fullscreen );
+
     SDL_WM_SetCaption( "LTris", 0 );
     sdl.fade = config.fade;
     SDL_SetEventFilter( event_filter );
